import java.awt.datatransfer.DataFlavor;  
import java.awt.datatransfer.Transferable;  
import java.awt.datatransfer.UnsupportedFlavorException;  
import java.awt.dnd.DnDConstants;  
import java.awt.dnd.DropTarget;  
import java.awt.dnd.DropTargetDragEvent;  
import java.awt.dnd.DropTargetDropEvent;  
import java.awt.dnd.DropTargetEvent;  
import java.awt.dnd.DropTargetListener;  
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.MouseInfo;
import java.io.IOException;  
import java.io.File;  
import java.util.*;
import java.awt.TextField;
import javax.swing.*;
import java.awt.Frame;
import java.awt.Point;
//import java.awt.BorderLayout;
//import java.awt.datatransfer.Clipboard;
//import java.awt.datatransfer.Transferable;
//import java.awt.datatransfer.DataFlavor;
//import java.awt.datatransfer.UnsupportedFlavorException;

PApplet app = null;
DropTarget dropTarget;
TextField field_width = new TextField("100"); 
TextField field_height = new TextField("100");
boolean isCapturing = false;

PImage src_img = null;
PImage output_img = null;
PImage bg_img = null;
float scale_factor = 0;
int corner_x = 0;
int corner_y = 0;
int scaled_width = 0;
int scaled_height = 0;
int cut_width = 0;
int cut_height = 0;

void setup() {
  size( 600, 400 );
  app = this;
  // src_img = loadImage("sample.jpg");
  bg_img = loadImage("drop_image_file_here.png");
  scale_factor = 1.0;

  setLayout(null);
  field_width.setBounds( 15, height-30, 50, 20 );
  field_height.setBounds( 80, height-30, 50, 20 );
  add(field_width);
  add(field_height);

  // drag-and-drop setting  
  dropTarget = new DropTarget(this, new DropEventListener() );
  frame.setResizable(true);
  frame.addComponentListener(new ComponentAdapter() {
    public void componentResized(ComponentEvent e) {
      if (e.getSource()==frame) {
        redraw();
      }
    }
  }
  );
}

void draw() {
  background(255);

  // if src image is not loaded, show bg_img and return
  if ( src_img == null ) {
    image( bg_img, 0, 0 );
    return;
  }

  // draw background chessboard
  drawChessboard();

  // set position of fields
  field_width.setBounds( 15, height-30, 50, 20 );
  field_height.setBounds( 80, height-30, 50, 20 );

  // get cut size from textfield
  cut_width = abs(int(field_width.getText().trim()));
  cut_height = abs(int(field_height.getText().trim()));

  // constrain scale factor by image size
  if ( scaled_width * 1.0/cut_width < 1.0 ) {
    scale_factor = cut_width * 1.0/src_img.width;
  }

  // calculate size of scaled source image
  scaled_width = round(src_img.width*scale_factor+0.5);
  scaled_height = round(src_img.height*scale_factor+0.5);

  // constrain corner position
  if ( src_img != null ) {
    corner_x = constrain( mouseX - (int)(cut_width/2.0), 0, scaled_width - cut_width );
    corner_y = constrain( mouseY - (int)(cut_height/2.0), 0, scaled_height - cut_height );
    /*
    if ( mousePressed ) {
     corner_x = mouseX - (int)(cut_width/2.0);
     corner_y = mouseY - (int)(cut_height/2.0);
     }
     corner_x = constrain( corner_x, 0, scaled_width - cut_width );
     corner_y = constrain( corner_y, 0, scaled_height - cut_height );
     */
  }

  // draw image
  image( src_img, 0, 0, scaled_width, scaled_height );

  // draw dark pixel
  loadPixels();
  for (int y=0; y<height; y++) {
    for (int x=0; x<width; x++) {
      int i = y*width + x;
      color c = pixels[i];
      // dark pixel
      if ( !( corner_x<=x && x<corner_x+cut_width && corner_y<=y && y<corner_y+cut_height) ) {
        pixels[i] = color( red(c)*0.2, green(c)*0.2, blue(c)*0.2 );
      }
      // draw background pixel
      if ( src_img != null ) {
        if ( x >= scaled_width || y >= scaled_height ) {
          pixels[i] = color(200);
        }
      }
    }
  }
  updatePixels();

  // save process
  if ( isCapturing ) {
    output_img = createImage( cut_width, cut_height, ARGB);
    if ( src_img != null ) {
      output_img.copy( src_img, (int)(corner_x/scale_factor), (int)(corner_y/scale_factor), 
      (int)(cut_width/scale_factor), (int)(cut_height/scale_factor), 
      0, 0, cut_width, cut_height );    // 0, 0, cut_width+1, cut_height+1 );
    }
    selectInput("Save image as...", "fileSelected");    
    isCapturing = false;
  }
}

// when mouse clicked, save image
void mouseClicked() {
  isCapturing = true;
}

// image scaling by mouse wheel
void mouseWheel(MouseEvent event) {
  float e = event.getCount();
  if ( e < 0 ) {
    scale_factor += 0.01;
  } else if ( e > 0 ) {
    scale_factor -= 0.01;
  }
}

// Chessboard
void drawChessboard() {
  for (int y=0; y<height/10+1; y++) {
    for (int x=0; x<width/10+1; x++) {
      noStroke();
      if ( (x+y)%2==0 ) {
        fill(200);
      } else {
        fill(255);
      }
      rect( x*10, y*10, 10, 10 );
    }
  }
}

// File dialog
void fileSelected(File selection) {
  if (selection == null) {
    // println("Window was closed or the user hit cancel.");
  } else {
    // println("User selected " + selection.getAbsolutePath());    
    String filename = selection.getAbsolutePath();
    File f = new File(filename);
    if ( f.exists() ) {
      String m = String.format("<html>%s already exists.<br>Do you want to replace it?", f.getAbsolutePath());
      int rv = JOptionPane.showConfirmDialog( this, m, "Save As", JOptionPane.YES_NO_OPTION);
      if ( rv == JOptionPane.NO_OPTION ) {
        return;
      }
    } 
    output_img.save(filename);
  }
}

// Drag-and-drop interaction
class DropEventListener implements DropTargetListener {
  public void dragEnter(DropTargetDragEvent dtde) {
  }  
  public void dragOver(DropTargetDragEvent dtde) {
  }  
  public void dropActionChanged(DropTargetDragEvent dtde) {
  }  
  public void dragExit(DropTargetEvent dte) {
  }  
  public void drop(DropTargetDropEvent dtde) {  
    dtde.acceptDrop(DnDConstants.ACTION_COPY_OR_MOVE);  
    Transferable trans = dtde.getTransferable();  
    List<File> fileNameList = null;  
    if (trans.isDataFlavorSupported(DataFlavor.javaFileListFlavor)) {  
      try {  
        fileNameList = (List<File>)  
          trans.getTransferData(DataFlavor.javaFileListFlavor);
      } 
      catch (UnsupportedFlavorException ex) {
      } 
      catch (IOException ex) {
      }
    }  
    if (fileNameList == null) return;

    for (int i=0; i<fileNameList.size (); i++) {
      String path = fileNameList.get(i).getAbsolutePath();
      src_img = loadImage(path);
      if ( src_img != null ) {
        app.requestFocus();
        scale_factor = 1.0;
        break;
      }
    }
  }
}

